@@ -1,8 +1,13 @@
 import uuid
 import jwt
+import inspect
+import os
+import time
+from pathlib import Path
 from flask import Flask, jsonify, request
 from flask.json import JSONEncoder
 from werkzeug.routing import BaseConverter
+from werkzeug import secure_filename
 from inspect import signature
 from .page import Page
 from .element import Element
@@ -104,14 +109,21 @@ class ErrorResponse(Element):
 
 class AdminApp:
     """Create an AdminUI App"""
-    SECRET = "admin ui super &*#*$ secret"
-    app_title = 'Admin UI App'
-    app_logo = None
-
-    def __init__(self):
+    def __init__(self, upload_folder=None):
+        self.app_title = 'Admin UI App'
+        self.app_logo = None
+        self.SECRET = "admin ui super &*#*$ secret"
         self.app = Flask(__name__, static_url_path='/')
         self.app.json_encoder = ElementJSONEncoder
         self.app.url_map.converters['purePath'] = PurePathConverter 
+        if upload_folder is None:
+            # the upload folder is not defined. using the main_module_path/upload as the folder
+            frame = inspect.stack()[1]
+            module = inspect.getmodule(frame[0])
+            upload_folder = os.path.join(os.path.dirname(os.path.abspath(module.__file__)), 'upload')
+        if not os.path.exists(upload_folder):
+            os.makedirs(upload_folder)
+        self.app.config['UPLOAD_FOLDER'] = upload_folder
         self.pages = {}
         self.menu = []
         self.on_login = {}
@@ -225,6 +237,27 @@ class AdminApp:
     def serve_root(self, path=''):
         """!!! Private method, don't call. Serve the index.html"""
         return self.app.send_static_file('index.html')
+    
+    def serve_upload(self):
+        """!!! Private method, don't call. Serve the upload endpoint"""
+        f = next(request.files.values())
+        Path(self.app.config['UPLOAD_FOLDER']).mkdir(parents=True, exist_ok=True) # create dir if not exist
+        filename = str(int(time.time() * 1000)) + '_' + secure_filename(f.filename) # create unique filename with timestamp prefix
+        f.save(os.path.join(self.app.config['UPLOAD_FOLDER'], filename))
+        return filename
+
+    def uploaded_file_name(self, uploaded_file):
+        if 'file_name' in uploaded_file:
+            return uploaded_file['file_name']
+        elif 'response' in uploaded_file:
+            return uploaded_file['response']
+        elif 'file' in uploaded_file:
+            return self.uploaded_file_name(uploaded_file['file'])
+        else:
+            return None
+
+    def uploaded_file_location(self, uploaded_file):
+        return os.path.join(self.app.config['UPLOAD_FOLDER'], self.uploaded_file_name(uploaded_file))
 
     def run(self):
         """run the AdminUI App"""
@@ -237,6 +270,7 @@ class AdminApp:
         self.app.route('/api/main_menu')(self.serve_menu)
         self.app.route('/api/app_settings')(self.serve_settings)
         self.app.route('/api/login', methods=['POST'])(self.handle_login_action)
+        self.app.route('/api/upload', methods=['POST'])(self.serve_upload)
         self.app.route('/api/page_action', methods=['POST'])(self.handle_page_action)
         self.app.route('/')(self.serve_root)
         self.app.route('/<purePath:path>/')(self.serve_root)
